<?php
/*******************************************************************
 *          
 *          Fichier         :   functions.stats.php 
 *          Cr le         :   22 octobre 2002 
 *          Dernire modif  :   09 novembre 2005 
 *          Email           :   wascripts@phpcodeur.net 
 * 
 *               Copyright 2002-2005 PHPcodeur
 * 
 *******************************************************************/

/*******************************************************************
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as 
 *  published by the Free Software Foundation; either version 2 of 
 *  the License, or (at your option) any later version. 
 *******************************************************************/


if( !defined('FUNCTIONS_STATS_INC') )
{

define('FUNCTIONS_STATS_INC', 1);
define('WA_STATS_PATH'      , $waroot . 'stats/', true);

/*
 * rvb_color()
 * 
 * Conversation de couleur hexadcimale  couleur rvb
 * 
 * @param resource $im      : Id de ressource de l'image
 * @param string   $color   : Couleur en hexadcimal
 * 
 * @return integer
 */
function rvb_color($im, $color)
{
    preg_match('/^([[:xdigit:]]{2})([[:xdigit:]]{2})([[:xdigit:]]{2})$/', strtoupper($color), $color_parts);
    
    return imagecolorallocate($im, hexdec($color_parts[1]), hexdec($color_parts[2]), hexdec($color_parts[3]));
}

/*
 * xy_arc()
 * 
 * Calcule les coordonnes du rayon
 * 
 * @param float   $degre    : Degr
 * @param integer $diametre : Diamtre du cercle
 * 
 * @return array
 */
function xy_arc($degre, $diametre)
{
    $x_arc = (cos($degre * (pi() / 180.0)) * ($diametre / 2));
    $y_arc = (sin($degre * (pi() / 180.0)) * ($diametre / 2));
    
    return array($x_arc, $y_arc);
}

/*
 * create_stats()
 * 
 * Cr le fichier de statistiques pour le mois et l'anne donns
 * 
 * @param array   $listdata : Donnes de la liste concerne
 * @param integer $month    : Chiffre du mois
 * @param integer $year     : Chiffre de l'anne
 * 
 * @return boolean
 */
function create_stats($listdata, $month, $year)
{
    global $db, $nl_config;
    
    if( $nl_config['disable_stats'] || !is_available_extension('gd') )
    {
        return false;
    }
    
    @set_time_limit(300);
    
    $filename = date('Y_F', mktime(0, 0, 0, $month, 1, $year)) . '_list' . $listdata['liste_id'] . '.txt';
    
    if( $fw = @fopen(wa_stats_path . $filename, 'w') )
    {
        $stats = array();
        
        for( $day = 1, $i = 0; $day <= 31; $day++, $i++ )
        {
            $stats[$i] = 0;
            
            if( checkdate($month, $day, $year) )
            {
                $min_time = mktime(0, 0, 0, $month, $day, $year);
                $max_time = mktime(23, 59, 59, $month, $day, $year);
                
                $sql = "SELECT COUNT(a.abo_id) AS num_abo 
                    FROM " . ABONNES_TABLE . " AS a, " . ABO_LISTE_TABLE . " AS al 
                    WHERE al.liste_id = " . $listdata['liste_id'] . " 
                        AND a.abo_id = al.abo_id 
                        AND a.abo_status = " . ABO_ACTIF . " 
                        AND ( a.abo_register_date BETWEEN $min_time AND $max_time )";
                if( $result = $db->query($sql) )
                {
                    $stats[$i] = $db->result($result, 0, 'num_abo');
                }
            }
        }
        
        fwrite($fw, implode("\n", $stats));
        fclose($fw);
        
        return true;
    }
    
    return false;
}

/*
 * update_stats()
 * 
 * Mise  jour des donnes pour les statistiques 
 * 
 * @param array $listdata   : Donnes de la liste concerne
 * 
 * @return boolean
 */
function update_stats($listdata)
{
    global $nl_config;
    
    if( $nl_config['disable_stats'] || !is_available_extension('gd') )
    {
        return false;
    }
    
    $filename = date('Y_F') . '_list' . $listdata['liste_id'] . '.txt';
    
    if( !file_exists(wa_stats_path . $filename) )
    {
        return create_stats($listdata, date('m'), date('Y'));
    }
    else
    {
        if( $fp = @fopen(wa_stats_path . $filename, 'r') )
        {
            $stats = clean_stats(fread($fp, filesize(wa_stats_path . $filename)));
            fclose($fp);
            
            $offset = (date('j') - 1);
            $stats[$offset] += 1;
            
            @chmod(wa_stats_path . $filename, 0666);
            if( $fw = @fopen(wa_stats_path . $filename, 'w') )
            {
                fwrite($fw, implode("\n", $stats));
                fclose($fw);
                
                return true;
            }
        }
        
        return false;
    }
}

/*
 * remove_stats()
 * 
 * Suppression/dplacement de stats (lors de la suppression d'une liste) 
 * 
 * @param integer $liste_from   : Id de la liste dont on supprime/dplace les stats
 * @param mixed   $liste_to     : Id de la liste de destination ou boolean (dans ce cas, on supprime)
 * 
 * @return boolean
 */
function remove_stats($liste_from, $liste_to = false)
{
    global $nl_config, $waroot;
    
    if( $nl_config['disable_stats'] || !is_available_extension('gd') )
    {
        return false;
    }
    
    @set_time_limit(300);
    
    if( $res = @opendir(wa_stats_path) )
    {
        include($waroot . 'includes/class.attach.php');
        
        $old_stats = array();
        
        while( $filename = @readdir($res) )
        {
            if( preg_match("/^([0-9]{4}_[a-zA-Z]+)_list$liste_from\.txt$/i", $filename, $match) )
            {
                if( $liste_to && $fp = @fopen(wa_stats_path . $filename, 'r') )
                {
                    $old_stats[$match[1]] = clean_stats(fread($fp, filesize(wa_stats_path . $filename)));
                    fclose($fp);
                }
                
                Attach::remove_file(wa_stats_path . $filename);
            }
        }
        closedir($res);
        
        if( $liste_to && count($old_stats) )
        {
            foreach( $old_stats AS $date => $stats_from )
            {
                $filename = $date . '_list' . $liste_to . '.txt';
                
                if( $fp = @fopen(wa_stats_path . $filename, 'r') )
                {
                    $stats_to = clean_stats(fread($fp, filesize(wa_stats_path . $filename)));
                    fclose($fp);
                    
                    for( $i = 0; $i < count($stats_to); $i++ )
                    {
                        $stats_to[$i] += $stats_from[$i];
                    }
                    
                    @chmod(wa_stats_path . $filename, 0666);
                    if( $fw = @fopen(wa_stats_path . $filename, 'w') )
                    {
                        fwrite($fw, implode("\n", $stats_to));
                        fclose($fw);
                    }
                }
            }
        }
        
        return true;
    }
    
    return false;
}

/*
 * clean_stats()
 * 
 * Effectue les traitements adquats sur la chaine et retourne un tableau
 * 
 * @param string $contents  : Contenu du fichier des statistiques
 * 
 * @return array
 */
function clean_stats($contents)
{
    $contents = preg_replace("/\r\n?/", "\n", trim($contents));
    
    return array_map('intval', explode("\n", $contents));
}

}
?>