<?php
/*******************************************************************
 *          
 *          Fichier         :   functions.validate.php 
 *          Cr le         :   10 janvier 2003 
 *          Dernire modif  :   29 fvrier 2004 
 *          Email           :   wascripts@phpcodeur.net 
 * 
 *              Copyright  2002-2004 phpCodeur
 * 
 *******************************************************************/

/*******************************************************************
 *  This program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as 
 *  published by the Free Software Foundation; either version 2 of 
 *  the License, or (at your option) any later version. 
 *******************************************************************/


if( !defined('FUNCTIONS_VALIDATE_INC') )
{

define('FUNCTIONS_VALIDATE_INC', true);

/*
 * check_email()
 * 
 * Vrification de l'email
 * 
 * @param string  $email    : Email  vrifier
 * @param integer $liste    : Id de la liste concerne
 * @param string  $action   : Action en cours
 * 
 * @return array
 */
function check_email($email, $liste = 0, $action = '', $disable_check_mx = false)
{
    global $waroot, $db, $nl_config, $lang;
    
    if( !class_exists('Mailer') )
    {
        include($waroot . 'includes/class.mailer.php');
    }
    
    //
    // Vrification syntaxique de l'email
    //
    $result = Mailer::validate_email($email);
    
    if( !$result )
    {
        return array('error' => true, 'message' => $lang['Message']['Invalid_email']);
    }
    
    $row = '';
    if( $liste > 0 )
    {
        $sql = 'SELECT ban_email FROM ' . BANLIST_TABLE . ' 
            WHERE liste_id = ' . $liste;
        if( $result = $db->query($sql) )
        {
            while( $row = $db->fetch_array($result) )
            {
                if( preg_match('/\b' . str_replace('*', '.*?', $row['ban_email']) . '\b/i', $email) )
                {
                    return array('error' => true, 'message' => $lang['Message']['Email_banned']);
                }
            }
        }
        
        switch( DATABASE )
        {
            case 'postgre':
                $sql = "SELECT a.*, 1 AS is_registered 
                    FROM " . ABONNES_TABLE . " AS a, " . ABO_LISTE_TABLE . " AS al 
                    WHERE LOWER(a.abo_email) = '" . $db->escape(strtolower($email)) . "' 
                        AND a.abo_id = al.abo_id 
                        AND al.liste_id = $liste 
                        UNION(
                            SELECT a.*, 0 AS is_registered 
                            FROM " . ABONNES_TABLE . " AS a 
                            WHERE LOWER(a.abo_email) = '" . $db->escape(strtolower($email)) . "' 
                                AND NOT EXISTS(
                                    SELECT abo_id 
                                    FROM " . ABO_LISTE_TABLE . " AS al 
                                    WHERE al.abo_id = a.abo_id 
                                        AND al.liste_id = $liste
                            )
                        )";
                break;
            
            default:
                $sql = "SELECT a.*, COUNT(al.abo_id) AS is_registered 
                    FROM " . ABONNES_TABLE . " AS a 
                    LEFT JOIN " . ABO_LISTE_TABLE . " AS al ON a.abo_id = al.abo_id 
                        AND al.liste_id = $liste 
                    WHERE LOWER(a.abo_email) = '" . $db->escape(strtolower($email)) . "' 
                    GROUP BY al.abo_id";
                break;
        }
        
        if( !($result = $db->query($sql)) )
        {
            trigger_error('Impossible de tester les tables d\'inscriptions', ERROR);
        }
        
        if( $row = $db->fetch_array($result) )
        {
            if( $row['is_registered'] )
            {
                if( $action == 'inscription' )
                {
                    return array('error' => true, 'message' => $lang['Message']['Allready_reg']);
                }
            }
            else
            {
                if( $action == 'desinscription' || $action == 'setformat' )
                {
                    return array('error' => true, 'message' => $lang['Message']['Unknown_email']);
                }
            }
            
            if( $action == 'confirmation' && $row['abo_status'] == ABO_ACTIF )
            {
                return array('error' => true, 'message' => $lang['Message']['Allready_confirm']);
            }
        }
        else if( $action != 'inscription' )
        {
            return array('error' => true, 'message' => $lang['Message']['Unknown_email']);
        }
    }
    
    if( !$disable_check_mx && $nl_config['check_email_mx'] && ( !$liste || !is_array($row) ) )
    {
        //
        // Vrification de l'existence d'un Mail eXchanger sur le domaine de l'email, 
        // et vrification de l'existence du compte associ (La vrification de l'existence du 
        // compte n'est toutefois pas infaillible, les serveurs smtp refusant parfois le relaying, 
        // c'est  dire de traiter les demandes manant d'un entit extrieure  leur rseau, et 
        // pour une adresse email extrieure  ce rseau)
        //
        $mailer = new Mailer();
        $mailer->smtp_path = $waroot . 'includes/';
        $result = $mailer->validate_email_mx($email, $err);
        
        if( !$result )
        {
            return array('error' => true, 'message' => $lang['Message']['Unrecognized_email']);
        }
    }
    
    return array('error' => false, 'abo_data' => $row);
}

function validate_pseudo($pseudo)
{
    return ( strlen($pseudo) >= 2 && strlen($pseudo) <= 30 );
}

function validate_pass($password)
{
    return preg_match('/^[\x21-\x7E]{4,32}$/', $password);
}

function validate_lang($language)
{
    global $waroot;
    
    return preg_match('/^[\w_-]+$/', $language) && file_exists($waroot . 'language/lang_' . $language . '.php');
}

}
?>